use std::env;
use std::fs::{copy, create_dir_all};
use std::path::PathBuf;
use std::sync::mpsc::channel;

use js_proxy_gen::{generate_proxy_crate, parse_crates, spawn};

/*
* 默认的代理库路径
*/
#[cfg(target_os = "windows")]
const DEFAULT_PI_JS_PROXY_CRATE_PATH: &str = r#"..\pi_serv_ext"#;
#[cfg(target_os = "linux")]
const DEFAULT_PI_JS_PROXY_CRATE_PATH: &str = "../pi_serv_ext";

/*
* 默认的代理库版本
*/
const DEFAULT_PI_JS_PROXY_CRATE_VERSION: &str = "0.1.0";

/*
* 默认的代理库Rust版本
*/
const DEFAULT_PI_JS_PROXY_CRATE_EDITION: &str = "2018";

/*
* 默认的分析模式为顺序
*/
const DEFAULT_PI_JS_PROXY_PARSE_MODE: bool = false;

fn main() {
    //构建代理代码
    let ext_crates = if let Some(ext_crates_var) = env::var_os("PI_JS_PROXY_EXT_CRATES") {
        ext_crates_var
            .to_str()
            .unwrap()
            .split(';')
            .map(|path| PathBuf::from(path))
            .collect::<Vec<PathBuf>>()
    } else {
        panic!("Require set for PI_JS_PROXY_EXT_CRATES");
    };
    let crate_path = if let Some(crate_path_var) = env::var_os("PI_JS_PROXY_CRATE_PATH") {
        let crate_path = crate_path_var.to_str().unwrap();
        PathBuf::from(crate_path)
    } else {
        PathBuf::from(DEFAULT_PI_JS_PROXY_CRATE_PATH)
    };
    let crate_version = if let Some(crate_version_var) = env::var_os("PI_JS_PROXY_CRATE_VERSION") {
        crate_version_var.to_str().unwrap().to_string()
    } else {
        DEFAULT_PI_JS_PROXY_CRATE_VERSION.to_string()
    };
    let crate_edition = if let Some(crate_edition_var) = env::var_os("PI_JS_PROXY_CRATE_EDITION") {
        crate_edition_var.to_str().unwrap().to_string()
    } else {
        DEFAULT_PI_JS_PROXY_CRATE_EDITION.to_string()
    };
    let ts_path = if let Some(ts_path_var) = env::var_os("PI_JS_PROXY_TS_PATH") {
        let ts_path = ts_path_var.to_str().unwrap();
        PathBuf::from(ts_path)
    } else {
        panic!("Require set for PI_JS_PROXY_TS_PATH");
    };
    let is_concurrent =
        if let Some(is_concurrent_var) = env::var_os("PI_JS_PROXY_PARSE_IS_CONCURRENT") {
            is_concurrent_var
                .to_str()
                .unwrap()
                .parse::<bool>()
                .unwrap_or(DEFAULT_PI_JS_PROXY_PARSE_MODE)
        } else {
            DEFAULT_PI_JS_PROXY_PARSE_MODE
        };

    let (sender, receiver) = channel();
    spawn(async move {
        match parse_crates(ext_crates, None, is_concurrent).await {
            Err(e) => panic!("Parse ext crates failed, reason: {:#?}", e),
            Ok(crates) => {
                if let Err(e) = generate_proxy_crate(
                    crate_path,
                    ts_path,
                    PathBuf::from("../pi_v8/vm_builtin"),
                    crate_version.as_str(),
                    crate_edition.as_str(),
                    is_concurrent,
                    crates,
                )
                .await
                {
                    panic!("Generate proxy crate failed, reason: {:#?}", e);
                } else {
                    sender.send(true);
                }
            }
        }
    });

    if let Err(e) = receiver.recv() {
        panic!("Generate proxy crate failed, reaosn: {:?}", e);
    }
}
